/*
 * 
 * P2P anonymous BBS system zigumo
 * website:http://www.zigumo.org/
 * e-mail:hoge@zigumo.org
 *
 * Copyright (C) 2002 hoge-taicho
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * $Id: SendHandShakeCommand.cs,v 1.7 2003/04/25 03:06:04 hoge-taicho Exp $
 * 
 */

using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;

using Zigumo.MessageObject;
using Zigumo.Model;
using Zigumo.Network;

namespace Zigumo.CommandObject
{
	/// <summary>
	/// nhVFCNR}h
	/// </summary>
	public class SendHandShakeCommand : Zigumo.CommandObject.ControllerCommand
	{
		/// <summary>
		/// nhVFCNT[og
		/// </summary>
		private Servent servent;
		/// <summary>
		/// RlNV
		/// </summary>
		private TcpConnection connection = null;
		/// <summary>
		/// Xg[
		/// </summary>
		private NetworkStream networkStream = null;

		public SendHandShakeCommand(Servent servent)
		{
			this.servent = servent;
		}

		/// <summary>
		/// nhVFCN
		/// </summary>
		public override void Execute() {
			//ڑ
			try {
				connection = new TcpConnection(servent.Address.ToString(), servent.Port);
				networkStream = connection.GetStream();
			} catch(ArgumentNullException e) {
				Console.WriteLine(e);
				Console.WriteLine(e.StackTrace);
				model.Log.Write(string.Format("HandShake Connection failed to {0}:{1}", servent.Address.ToString(), servent.Port.ToString()));
				return;
			} catch(ArgumentOutOfRangeException e) {
				Console.WriteLine(e);
				Console.WriteLine(e.StackTrace);
				model.Log.Write(string.Format("HandShake Connection failed to {0}:{1}", servent.Address.ToString(), servent.Port.ToString()));
				return;
			} catch(SocketException e) {
				Console.WriteLine(e);
				Console.WriteLine(e.StackTrace);
				model.Log.Write(string.Format("HandShake Connection failed to {0}:{1}", servent.Address.ToString(), servent.Port.ToString()));
				return;
			} catch(InvalidOperationException e) {
				Console.WriteLine(e);
				Console.WriteLine(e.StackTrace);
				model.Log.Write(string.Format("HandShake Connection failed to {0}:{1}", servent.Address.ToString(), servent.Port.ToString()));
				return;
			}

			try {
				//nhVFCNM
				SendHandShake();

				//nhVFCNX|XM
				ReceiveHandShakeResponse();

				//P2PRlNVɒǉ
				AddP2PConnection();
			} catch(Exception e) {
				Console.WriteLine(e);
				Console.WriteLine(e.StackTrace);
				//ڑs
				model.Log.Write(string.Format("Send HandShake error to {0}:{1}", servent.Address.ToString(), servent.Port));

				networkStream.Close();
				try {
					connection.Close();
				} catch(SocketException) {
					Console.WriteLine(e);
					Console.WriteLine(e.StackTrace);
				}
			}
		}

		/// <summary>
		/// nhVFCNbZ[W𑗐M
		/// </summary>
		private void SendHandShake() {
			model.Log.Write(string.Format("Send HandShake to {0}:{1}", servent.Address.ToString(), servent.Port));

			string msg = "zigumo connect/" + model.ZigumoProtocolversion + "\r\n\r\n";
			byte[] handShakeMessage = Encoding.ASCII.GetBytes(msg);

			networkStream.Write(handShakeMessage, 0, handShakeMessage.Length);
		}

		/// <summary>
		/// nhVFCNM
		/// </summary>
		private void ReceiveHandShakeResponse() {
			model.Log.Write(string.Format("Receive HandShake response from {0}:{1}", servent.Address.ToString(), servent.Port));

			//nhVFCNԐMbZ[WM
			bool error = false;
			StringBuilder headerByteArray = new StringBuilder();
			byte[] data = new byte[1];
			byte[] keep_data = new Byte[4];
			int size;
			while(true) {
				try {
					if(networkStream.CanRead) {
						size = networkStream.Read(data, 0, 1);
						if(connection.Socket.Blocking && size == 0) {
							//ubLOȂ̂ɂ炸Asize==0ŃX[ĂIB
							//{ubLȌꍇAsize==0΃X[Ȃ̂Aꕔ̏n
							//Ƃ̒ʐMŁÂ悤ȌۂBperlsocketW[B
							model.Log.Write("zigumo Blocking socket through error");
							error = true;
							break;
						}
					} else {
						error = true;
						break;//ǂݍ݂łȂȂ烋[v𔲂
					}
				} catch(Exception e) {
					Console.WriteLine(e);
					Console.WriteLine(e.StackTrace);
					error = true;
					break;//O̓[v𔲂
				}
				headerByteArray.Append(Encoding.ASCII.GetString(data));// += Encoding.ASCII.GetString(data);

				//\r\n\r\n鏈
				keep_data[0] = keep_data[1];
				keep_data[1] = keep_data[2];
				keep_data[2] = keep_data[3];
				keep_data[3] = data[0];

				if(keep_data[0] == (byte)'\r' && keep_data[1] == (byte)'\n' && keep_data[2] == (byte)'\r' && keep_data[3] == (byte)'\n') {
					string header = headerByteArray.ToString();
			
					model.Log.Write(string.Format("Receive HandShake response header from {0}:{1}", servent.Address.ToString(), servent.Port));
					model.Log.Write("\r\n" + header);
					break;
				}
			}

			//wb_[擾G[
			if(error) {
				model.Log.Write(string.Format("Receive HandShake response header error from {0}:{1}", servent.Address.ToString(), servent.Port));
				throw new HandShakeException("nhVFCNX|X̎MɎs܂B");
			}
		}

		/// <summary>
		/// P2PRlNVVɒǉ
		/// </summary>
		private void AddP2PConnection() {
			//P2PRlNVVKɒǉ
			P2PConnection p2pCon = connectionManager.AddConnection(new P2PConnection(connection, networkStream));

			//pingbZ[W
			PingMessage ping = new PingMessage();
			ping.CreateHeader();

			//̑bZ[WLbVɓo^
			connectionManager.PutCache(ping.MessageIDToGuid, new ConnectionManager.MessageCacheData(ping.Type, p2pCon.ID, true));

			//bZ[WM
			connectionManager.SendMessage(ping, p2pCon, true);
			p2pCon.StartThread();
		}
	}
}
