/*
 * 
 * P2P anonymous BBS system zigumo
 * website:http://www.zigumo.org/
 * e-mail:hoge@zigumo.org
 *
 * Copyright (C) 2002 hoge-taicho
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * $Id: Message.cs,v 1.5 2003/04/25 03:06:38 hoge-taicho Exp $
 * 
 */

using System;
using System.Net;

namespace Zigumo.MessageObject
{
	/// <summary>
	/// bZ[WNX
	/// 0-15	16	bZ[WIDiGUIDj
	/// 16		 1	Type(int)
	/// 17		 1	TTL(int)
	/// 18		 1	Hops(int)
	/// 19-22	 4	yC[hTCY(int)
	/// ------------------------------------
	/// 23-			yC[h
	/// </summary>
	public class Message
	{
		/// <summary>
		/// f[^{
		/// </summary>
		protected byte[] data;

		/// <summary>
		/// wb_[̃TCY
		/// </summary>
		public const int headerSize = 23;

		public Message() {
		}

		public Message(int length) {
			data = new byte[length];
		}

		public Message(byte[] dat) {
			data = dat;
		}

		/// <summary>
		/// f[^ւ̃vpeB
		/// </summary>
		public virtual byte[] Data {
			get {
				return data;
			}
			set {
				data = value;
			}
		}
		/// <summary>
		/// bZ[WID̃vpeB
		/// </summary>
		public byte[] MessageID {
			get {
				if(data == null)
					throw new MessageNotInitializeException();

				byte[] id = new byte[16];
				Array.Copy(data, 0, id, 0, 16);
				return id;
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();
				
				Array.Copy(((byte[])value), 0, data, 0, 16);
			}
		}
		/// <summary>
		/// bZ[WIDGUIDNXƂĈvpeB
		/// </summary>
		public Guid MessageIDToGuid {
			get {
				if(data == null)
					throw new MessageNotInitializeException();

				byte[] guid = new byte[16];
				Array.Copy(data, 0, guid, 0, 16);
				return new Guid(guid);
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();

				byte[] guid = ((Guid)value).ToByteArray();
				guid.CopyTo(data, 0);
			}
		}
		/// <summary>
		/// bZ[W^CṽvpeB
		/// </summary>
		public byte Type {
			get {
				if(data == null)
					throw new MessageNotInitializeException();

				return (byte)data[16];
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();

				data[16] = value;
			}
		}
		/// <summary>
		/// TTL̃vpeB
		/// </summary>
		public int TTL {
			get {
				if(data == null)
					throw new MessageNotInitializeException();

				return (int)data[17];
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();

				data[17] = (byte)value;
			}
		}
		/// <summary>
		/// Hops̃vpeB
		/// </summary>
		public int Hops {
			get {
				if(data == null)
					throw new MessageNotInitializeException();

				return (int)data[18];
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();

				data[18] = (byte)value;
			}
		}
		/// <summary>
		/// yC[hTCỸvpeB
		/// </summary>
		public int PayloadSize {
			get {
				if(data == null)
					throw new MessageNotInitializeException();

				return IPAddress.NetworkToHostOrder(BitConverter.ToInt32(data, 19));
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();

				BitConverter.GetBytes(IPAddress.HostToNetworkOrder((int)value)).CopyTo(data, 19);
			}
		}
		/// <summary>
		/// data.Lengthւ̃vpeB
		/// </summary>
		public int Length {
			get {
				return data.Length;
			}
		}
		/// <summary>
		/// dataւ̃CfNT
		/// </summary>
		public virtual byte this[int index] {
			get {
				if(data == null)
					throw new MessageNotInitializeException();
				
				return data[index];
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();

				data[index] = value;
			}
		}
		/// <summary>
		/// Equals\bh
		/// </summary>
		/// <param name="message">rΏƂƂȂ郁bZ[W</param>
		/// <returns>ȂtrueႤȂfalse</returns>
		public bool Equals(Message message) {
			if(data == null)
				throw new MessageNotInitializeException();

			for(int i = 0; i < 16; i++)
				if(data[i] != message[i])
					return false;
			return true;
		}
		/// <summary>
		/// GUID𐶐郁\bh
		/// ̃\bhsMessageIDGUIDݒ肳
		/// </summary>
		public void CreateID() {
			if(data == null)
				throw new MessageNotInitializeException("bZ[WĂ܂B");

			byte[] guid = Guid.NewGuid().ToByteArray();
			guid.CopyTo(data, 0);
		}
		/// <summary>
		/// wb_[𐶐郁\bh
		/// GUID𐶐ATTL=8AHops=0ɏB
		/// </summary>
		public virtual void CreateHeader() {
			this.CreateID();
			this.TTL = 7;
			this.Hops = 0;
		}

	}

	/// <summary>
	/// bZ[WĂȂꍇɂłO
	/// </summary>
	public class MessageNotInitializeException : ApplicationException {
		public MessageNotInitializeException() {
		}
		public MessageNotInitializeException(string message) : base(message) {
		}
		public MessageNotInitializeException(string message, Exception inner) : base(message, inner) {
		}
	}

	/// <summary>
	/// bZ[W^Cv\Enum
	/// </summary>
	public enum MessageType {
		Ping		= 0x00,
		Pong		= 0x01,
		Query		= 0x80,
		QueryHit	= 0x81,
		QueryChat	= 0x90,
		QueryChatHit= 0x91,
		Push		= 0x40,
	}

	/// <summary>
	/// T[ogXe[^X킷Enum
	/// </summary>
	public enum ServentStatus {
		NormalServent = 0,
		ChatServer = 1,
	}
}
