/*
 * 
 * P2P anonymous BBS system zigumo
 * website:http://www.zigumo.org/
 * e-mail:hoge@zigumo.org
 *
 * Copyright (C) 2002 hoge-taicho
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * $Id: PongMessage.cs,v 1.4 2003/04/24 18:13:04 hoge-taicho Exp $
 * 
 */

using System;
using System.Net;
using System.Net.Sockets;

namespace Zigumo.MessageObject
{
	/// <summary>
	/// pongbZ[WNX
	/// 0-15	16	bZ[WIDiGuidj
	/// 16		 1	Description(int)
	/// 17		 1	TTL(int)
	/// 18		 1	Hops(int)
	/// 19-22	 4	yC[hTCY(int)
	/// ------------------------------------
	/// 23-24	 2	|[gԍ(int)
	/// 25-28	 4	IPAhX(IPAddress)
	/// 29-32	 4	Xbh(int)
	/// 33		 1	T[ogXe[^X(int)
	/// </summary>
	public class PongMessage : Zigumo.MessageObject.Message
	{
		/// <summary>
		/// pongbZ[WyC[hTCY
		/// </summary>
		public const int pongPayloadSize = 11;

		public PongMessage() : this(Message.headerSize + PongMessage.pongPayloadSize) {
		}

		public PongMessage(int length) : base(length) {
		}

		public PongMessage(byte[] data) : base(data) {
		}	
		/// <summary>
		/// |[gԍ̃vpeB
		/// </summary>
		public int Port {
			get {
				if(data == null)
					throw new MessageNotInitializeException();

				return (int)IPAddress.NetworkToHostOrder(BitConverter.ToInt16(data, 23));
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();

				BitConverter.GetBytes(IPAddress.HostToNetworkOrder((Int16)value)).CopyTo(data, 23);
			}
		}
		/// <summary>
		/// IPAhX̃vpeB
		/// </summary>
		public IPAddress Address {
			get {
				if(data == null)
					throw new MessageNotInitializeException();

				return new IPAddress(BitConverter.ToInt32(data, 25));
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();

				((IPAddress)value).GetAddressBytes().CopyTo(data, 25);
			}
		}
		/// <summary>
		/// Xbh̃vpeB
		/// </summary>
		public int ThreadCount {
			get {
				if(data == null)
					throw new MessageNotInitializeException();

				return IPAddress.NetworkToHostOrder(BitConverter.ToInt32(data, 29));
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();

				BitConverter.GetBytes(IPAddress.HostToNetworkOrder((int)value)).CopyTo(data, 29);
			}
		}
		/// <summary>
		/// Xe[^X̃vpeB
		/// </summary>
		public int Status {
			get {
				if(data == null)
					throw new MessageNotInitializeException();

				return (int)data[33];
			}
			set {
				if(data == null)
					throw new MessageNotInitializeException();

				data[33] = (byte)value;
			}
		}
		/// <summary>
		/// wb_[\bh
		/// </summary>
		public override void CreateHeader() {
			base.CreateHeader();
			this.Type = (byte)MessageType.Pong;
			this.PayloadSize = 11;
		}

		/// <summary>
		/// bZ[W̖OԂ\bh
		/// </summary>
		/// <returns>bZ[W</returns>
		public override string ToString() {
			return "Pong";
		}
	}
}
